/**********************************************************************
 * SimMips: Simple Computer Simulator of MIPS                         *
 * Copyright (C) 2008-2012 Arch Lab. Tokyo Institute of Technology    *
 *  Dual licensed under New BSD or GPL v2 licenses.                   *
 *  See COPYING file on the base directory for more information.      *
 **********************************************************************/
#include "syscallplus.h"

SystemCallEmulator sc;

/**********************************************************************/
void Mips::syscall()
{
    if (as->r[REG_V0] == SYS_EXIT) {
        state = CPU_STOP;
        return;
    }
    if (! sc.initialized)
        sc.init(mc, ss->heapptr, as->r[REG_SP]);
    sc.silent_mode = chip->issilent();
        
    sc.syscall(as->r, chip->cycle);
}

/**********************************************************************/
void MipsSimstate::writesnap(FILE *snap) {
    std::vector<std::vector<uint032_t> > &log = sc.getlog();
    fprintf(snap, "[SYSCALL]\n");
    for (uint i = 0; i < log.size(); i++) {
        fprintf(snap, "%08x %08x\n",
                log[i][0], (uint) log[i].size() - 1);
        for (uint j = 0; j < log[i].size() - 1; j++)
            fprintf(snap, "%08x%c", log[i][j + 1],
                    (j % 8 == 7) ? '\n' : ' ');
        if (log[i].size() % 8 != 1)
            fprintf(snap, "\n");
    }
    fprintf(snap, "00000000\n");
}

/**********************************************************************/
